




# predict

# -*- coding: utf-8 -*-
import xml.dom.minidom
from vcvf import face_detector as fd
import cv2
import os
from roc import *


def predict(img_dir,reusltdir):

    fd1=fd.face_detector(1)
    files1 =os.listdir(img_dir)
    for xmlFile in files1:
        list1=[]
        name=xmlFile[:-4]
        imf=os.path.join(img_dir,xmlFile)
        print imf
        image=cv2.imread(imf)

        num,list1,time=fd1.detect_face(image)

        pathn=os.path.join(resultdir,xmlFile)
        pathn=pathn[:-4]+".xml"
        txtfile="tmp.txt"
        with open(txtfile, 'w') as fp:
            fp.write("%s %d " % (name, num))
            for i in range(num):
                list2=str(list1[i])
                list2=list2[1:-1]
                print list2
                fp.write("%s " % list2)
        xmlfilename=pathn
        txt2xml(txtfile,xmlfilename) 

def txt2xml(xmlfilename,txtfilename):
    with open(xmlfilename,'r') as fs:
            managerList=[]
            doc = xml.dom.minidom.Document()
            root = doc.createElement('Recognition')
            root.setAttribute('type', 'face')
            doc.appendChild(root)

            str=fs.read()
            line=str.split()
            print line
            for i in range(0,int(line[1])):
                managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])

            for i in managerList :
                for j in range(len(i)):
                    nodeManager = doc.createElement('bndbox')
                    nodeXmin = doc.createElement("xmin")
                    nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                    nodeYmin = doc.createElement("ymin")
                    nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                    nodeXmax = doc.createElement("xmax")
                    nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                    nodeYmax = doc.createElement("ymax")
                    nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))


                    nodeManager.appendChild(nodeXmin)
                    nodeManager.appendChild(nodeYmin)
                    nodeManager.appendChild(nodeXmax)
                    nodeManager.appendChild(nodeYmax)
                    root.appendChild(nodeManager)

            print line[0]
            #pathn=os.path.join(after_file,line[0])
            #pathn=os.path.join(pathn,".xml")
            #pathn+=".xml"
            #fp = open(pathn, 'w')
            fp = open(txtfilename, 'w')
            doc.writexml(fp, indent='\t', addindent='\t', newl='\n', encoding="utf-8")

# caculate
if __name__ == '__main__':
    img_dir ="D:\zmm\moba\home\intern_mission\\bndbox_test2\\bndbox_test\\vcvf_txt\pre_file"
    resultdir="D:\zmm\moba\home\intern_mission\\bndbox_test2\caculate_roc\\predict_result1"
    gt_dir="D:\\zmm\\moba\\home\\intern_mission\\bndbox_test2\\caculate_roc\\gt"
    #predict(img_dir,resultdir)
    roc(gt_dir,resultdir,'result_roc.txt',"result_IOU.txt")
    



# -*- coding: utf-8 -*-
import pylab as pl
from Analyze_xml import Analyze_xml
import sys
def roc(standard_path="truth",test_path="test",result_roc='result_roc.txt'):

    db=[]
    print "analyze_xml:"
    db,pos=Analyze_xml(standard_path,test_path)
    db = sorted(db, key=lambda x:x[2], reverse=True)#sorted() 
    
    print pos 
    xy_arr = []
    tp,fp = 0., 0.			
    for i in range(len(db)):
        #print db
        tp += db[i][1]#wrong
        fp += db[i][0]#creat
        xy_arr.append([tp,fp/pos,db[i][2]])
   

    auc = 0.			
    prev_x = 0
    for x,y,t in xy_arr:
	    if x != prev_x:
		    auc += (x - prev_x) * y
		    prev_x = x
            
          
    x = [_a[0] for _a in xy_arr]
    y = [_a[1] for _a in xy_arr]
    pl.title("ROC (AUC = %.4f)" % (auc/float(x[-1])))
    pl.xlabel("False Count")
    pl.ylabel("True Positive Rate")
    pl.plot(x, y)
    pl.show()

    #with open(result_roc, 'w') as fp:
    #    for i in range(len(db)):
    #        fp.write("%d %f \n" % (x[i], y[i]))
    db=xy_arr
    with open(result_roc, 'w') as fp:
        for i in range(len(db)):
            fp.write("%d %d %f \n" % (db[i][0], db[i][1],db[i][2]))


def Analyze_score_vector(score_vector):
    '''
    score_vector
    1 0.8
    -1 0.3
    1 0.45
    ...
    -1 0.3

    '''
    db=[]
    pos=0
    with open(score_vector) as fh:
        for lines in fh.readlines():
            line=lines.strip('\n').split(' ')
            print line
            score=float(line[1])
            sample_type=float(line[0])
            if float(sample_type)==1:
                db.append([1,0,score])
                pos+=1
            else:
                db.append([0,1,score])
    return db,pos

def roc_score_matrix(score_vector="score_vector.txt",result_roc='result_roc.txt',result_IOU='result_IOU'):

    db=[]
    print "analyze_xml:"
    #db,pos=Analyze_xml(standard_path,test_path)
    db,pos=Analyze_score_vector(score_vector)
    db = sorted(db, key=lambda x:x[2], reverse=True)#sorted() 
    
    print pos 
    xy_arr = []
    tp,fp = 0., 0.			
    for i in range(len(db)):
        #print db
        tp += db[i][1]#wrong
        fp += db[i][0]#creat
        xy_arr.append([tp,fp/pos,db[i][2]])
   

    auc = 0.			
    prev_x = 0
    for x,y,t in xy_arr:
	    if x != prev_x:
		    auc += (x - prev_x) * y
		    prev_x = x
          
    x = [_a[0] for _a in xy_arr]
    y = [_a[1] for _a in xy_arr]
    pl.title("ROC (AUC = %.4f)" % (auc/float(x[-1])))
    pl.xlabel("False Count")
    pl.ylabel("True Positive Rate")
    pl.plot(x, y)
    pl.show()
    with open(result_roc, 'w') as fp:
        for i in range(len(db)):
            fp.write("%d %f \n" % (x[i], y[i]))
    db=xy_arr
    with open(result_IOU, 'w') as fp:
        for i in range(len(db)):
            fp.write("%d %d %f \n" % (db[i][0], db[i][1],db[i][2]))
    
def main_1(gt_dir,resultdir):
    #gt_dir=sys.argv[1]
    #resultdir=sys.argv[2]
    print gt_dir
    print resultdir
    roc(gt_dir,resultdir,'result_roc.txt')

def main_2():
    roc_score_matrix("score_vector.txt","result_roc.txt")


if __name__ == '__main__':
    gt_dir=sys.argv[1]
    resultdir=sys.argv[2]
    main_1(gt_dir,resultdir)



# -*- coding: utf-8 -*-
import xml.dom.minidom 
import os.path
from read import read
from IOU import IOU
import os


def Analyze_xml(standard_path,test_path):

    wrong,creat,pos=0,0,0
    ratio=0.
    list1=[]
    Reframe=GTframe=[]
    path1=standard_path
    path2=test_path
    files1 = os.listdir(path2)
    cc1=0
    cc2=0
    for xmlFile in files1:
        
        testpath_xml=os.path.join(path2,xmlFile)
        standpath_xml=testpath_xml.replace(path2,path1)
        print standpath_xml
        print testpath_xml
        if os.path.isfile(standpath_xml):
            dom_test=xml.dom.minidom.parse(testpath_xml)
            dom_stand=xml.dom.minidom.parse(standpath_xml)
            root1=dom_test.documentElement
            root2=dom_stand.documentElement
            #print root2
            #get_char()
            Reframe=read(root1)
            GTframe=read(root2)
            #print GTframe
            pos+=len(GTframe)
            cc1+=1
            if len(GTframe)>len(Reframe):
                for i in range(len(GTframe)-len(Reframe)):
                    list1.append([0,1,0])
                    
            for i in range(len(Reframe)):
                wrong=creat=0
                for j in range(len(GTframe)):
                    ratio=IOU(Reframe[i],GTframe[j])
                    score=Reframe[i][4]
                    if ratio>=0.5:
                        creat=1
                        list1.append([creat,wrong,ratio])
                        break
                if creat==0:
                    wrong=1
                    list1.append([creat,wrong,ratio])
                
        else:
            print "file not exist!"
            cc2+=1
            creat=wrong=0
            #print testpath_xml
            dom_test=xml.dom.minidom.parse(testpath_xml)
            root1=dom_test.documentElement
            Reframe=read(root1)
            for i in range(len(Reframe)):
                ratio=0
                wrong=1
                list1.append([creat,wrong,ratio])
        print cc1,cc2
    return list1,pos
            
        



# -*- coding: utf-8 -*-
import xml.dom.minidom 
import os.path
from read import read,readgt
from IOU import IOU
import os
def txt2xml(txtfilename,xmlfilename):
    with open(txtfilename,'r') as fs:
            managerList=[]
            doc = xml.dom.minidom.Document()
            root = doc.createElement('Recognition')
            root.setAttribute('type', 'face')
            doc.appendChild(root)

            str=fs.read()
            line=str.split()
            print line
            for i in range(0,int(line[1])):
                #managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])
                managerList.append([{'xmin':line[3+i*5],'ymin':line[2+i*5],'xmax':line[5+i*5],'ymax':line[4+i*5],'score':line[6+i*5]}])

            for i in managerList :
                for j in range(len(i)):
                    nodeManager = doc.createElement('bndbox')
                    nodeXmin = doc.createElement("xmin")
                    nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                    nodeYmin = doc.createElement("ymin")
                    nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                    nodeXmax = doc.createElement("xmax")
                    nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                    nodeYmax = doc.createElement("ymax")
                    nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))
                    nodeScore = doc.createElement("score")
                    nodeScore.appendChild(doc.createTextNode(i[j]['score']))


                    nodeManager.appendChild(nodeXmin)
                    nodeManager.appendChild(nodeYmin)
                    nodeManager.appendChild(nodeXmax)
                    nodeManager.appendChild(nodeYmax)
                    nodeManager.appendChild(nodeScore)
                    root.appendChild(nodeManager)

            print line[0]
            #pathn=os.path.join(after_file,line[0])
            #pathn=os.path.join(pathn,".xml")
            #pathn+=".xml"
            #fp = open(pathn, 'w')
            fp = open(xmlfilename, 'w')
            doc.writexml(fp, indent='\t', addindent='\t', newl='\n', encoding="utf-8")


def Analyze_txt(standard_path,test_path):

    wrong,creat,pos=0,0,0
    ratio=0.
    list1=[]
    Reframe=GTframe=[]
    path1=standard_path
    path2=test_path
    files1 = os.listdir(path2)
    cc1=0
    cc2=0
    for xmlFile in files1:
        print xmlFile 
        #testpath_xml=os.path.join(path2,xmlFile)
        testpath_txt=os.path.join(path2,xmlFile)
        testpath_xml="tmp.xml"
        
        print testpath_txt
        print testpath_xml
        txt2xml(testpath_txt,testpath_xml)
        standpath_xml=testpath_txt.replace(path2,path1)
        standpath_xml=standpath_xml.replace(".txt",".xml")
        print 10*"*"
        print standpath_xml
        print testpath_xml
        if os.path.isfile(standpath_xml):
            dom_test=xml.dom.minidom.parse(testpath_xml)
            dom_stand=xml.dom.minidom.parse(standpath_xml)
            root1=dom_test.documentElement
            root2=dom_stand.documentElement
            #print root2
            #get_char()
            Reframe=read(root1)
            GTframe=readgt(root2)
            print GTframe
            print Reframe
            pos+=len(GTframe)
            cc1+=1
            if len(GTframe)>len(Reframe):
                for i in range(len(GTframe)-len(Reframe)):
                    list1.append([0,1,0])
                    
            for i in range(len(Reframe)):
                wrong=creat=0
                score=float(Reframe[i][4])
                print "score=%f"%(score)
                for j in range(len(GTframe)):
                    ratio=IOU(Reframe[i],GTframe[j])
                    if ratio>=0.5:
                        creat=1
                        list1.append([creat,wrong,score])
                        break
                if creat==0:
                    wrong=1
                    list1.append([creat,wrong,score])
                
        else:
            print "file not exist!"
            cc2+=1
            creat=wrong=0
            #print testpath_xml
            dom_test=xml.dom.minidom.parse(testpath_xml)
            root1=dom_test.documentElement
            Reframe=read(root1)
            for i in range(len(Reframe)):
                ratio=0
                wrong=1
                list1.append([creat,wrong,ratio])
        print cc1,cc2
    return list1,pos
            
            
if __name__=="__main__":
    txtfilename="Analyze_test.txt"
    xmlfilename="Analyze_test.xml"
    txt2xml(txtfilename,xmlfilename);



# -*- coding: utf-8 -*-

def IOU(Reframe=[],GTframe=[]):
    """
    计算两矩形 IOU，传入为均为矩形每条对角线，（x,y）  坐标。
    """
    #矩形框1
    x1 = Reframe[0]
    y1 = Reframe[1]
    width1 = Reframe[2]-Reframe[0]
    height1 = Reframe[3]-Reframe[1]
     
    #矩形框2
    x2 = GTframe[0]
    y2 = GTframe[1]
    width2 = GTframe[2]-GTframe[0]
    height2 = GTframe[3]-GTframe[1]

    endx = max(x1+width1,x2+width2)
    startx = min(x1,x2)
    width = width1+width2-(endx-startx)

    endy = max(y1+height1,y2+height2)
    starty = min(y1,y2)
    height = height1+height2-(endy-starty)

    if width <=0 or height <= 0:
        ratio = 0 # 重叠率为 0 
    else:
        Area = width*height # 两矩形相交面积
        Area1 = width1*height1
        Area2 = width2*height2
        ratio = Area*1./(Area1+Area2-Area)
    # return IOU
    print "IOU=%f"%(ratio)
    print Reframe
    print GTframe
    return ratio#,Reframe,GTframe



#!/usr/bin/python
import cv2

def save2server(imgname,img):
    ret=cv2.imwrite('/data1/mingmingzhao/data_sets/test/race_recog/'+imgname,img)
    ret=cv2.imwrite(imgname,img)
    print "local url is /data1/mingmingzhao/data_sets/test/race_recog/{}".format(imgname)
    print "remote url is http://192.168.7.37:8393/static/race_recog/{}".format(imgname)
    print "<img src='http://192.168.7.37:8393/static/race_recog/{}' title={} />".format(imgname,imgname)
    print "or u can view the image at :http://192.168.7.37:8393/show/?start_num=1&length=200&dirname=/{}".format(imgname.split('/')[:-1])
    return ret



#!/usr/bin/python
import cv2
from save2server import *
import numpy as np
import random
def norm(x,norm_range):
    '''
      normlize the value of x between the norm_range
      x: a numpy array
      norm_range : a numpy array, the first element is the low value,and the second is the high value
    '''
    x_=np.array(x)
    x_=(x_-x_.min())*(norm_range.max()-norm_range.min())/(x_.max()-x_.min())
    #print " input:{}".format(x)
    #print " range:{}".format(norm_range)
    #print "output:{}".format(x_)
    return x_
def putText(img,title,pos,font,scale,c,thinck_ness,direction='h'):
    '''
    write text on the image,use the opencv lib
      direction :  'h' mean write the text horizontally
                   'v' mean write the text vartically
                   'v1' mean write the text horizontally and rotate the image 90 degree
            img : 
          title : 
            pos :  position like (100,100)
           font :  open cv font,like 
          scale :  opencv scale
              c :  color like (255,0,255)
     think_ness : 
    '''
    #print c
    if direction=='h':
       npos=np.array(pos)
       width=npos[0]
       height=npos[1]
       npos=([int((width-len(title)*18*scale)/2), int(height/7*4)])
       pos=tuple(npos)
       cv2.putText(img,title,pos,font,scale,c,thinck_ness)
    elif direction=='v':
       npos=np.array(pos)
       width=npos[0]
       height=npos[1]
       npos=([int(width/7*4), int(height/len(title))])
       pos=tuple(npos)
       for x in title:
           npos=np.array(pos)
           npos[1]+=24*scale
           pos=tuple(npos)
           cv2.putText(img,x,pos,font,scale,c,thinck_ness)
    elif direction=='v1':
       #print 'img shape:{},dtype:{}'.format(img.shape,img.dtype)
       img_tmp=np.rot90(img,1,(1,0))
       #print 'img_tmp shape:{},dtype:{}'.format(img_tmp.shape,img_tmp.dtype)
       npos=np.array(pos)
       width=npos[1]
       height=npos[0]
       img_tmp=np.zeros((height,width,3),np.uint8)+255
       #npos=([int(width/len(title)), int(height/7*4)])
       npos=([int((width-len(title)*18*scale)/2), int(height/7*4)])
       pos=tuple(npos)
       #print pos
       cv2.putText(img_tmp,title,pos,font,scale,c,thinck_ness)
       #save2server('temp.jpg',img_tmp)
       img=np.rot90(img_tmp,1,(0,1))
       save2server('temp.jpg',img_tmp)
       img=np.rot90(img_tmp,1,(0,1))
       #print 'img shape:{},dtype:{}'.format(img.shape,img.dtype)
    return img 
def draw_text(title,width,height,text_scale='50',c=(10,10,10),direc='h'):
    img=np.zeros((height,width,3),np.uint8)+255
    font = cv2.FONT_HERSHEY_SIMPLEX
    #font=cv2.InitFont(cv2.FONT_HERSHEY_SIMPLEX)
    #c=(10,10,10)
    #print 'font.vscale={}'.format(font.vscale)
    #cv2.putText(img, title, (int(width/len(title)), int(height/7*4)), font, np.min([height,width])/70, c, 2)
    img=putText(img, title, (width,height), font, text_scale, c, 2,direc)
    return img 
def get_mark_value(y,num):
    ymax=float(y.max())
    ymin=float(y.min())
    num=float(num)
    print "ymin=%f,ymax=%f,interscale=%f"%(ymin,ymax,(ymax-ymin)/num)
    interscale=(ymax-ymin)/num
    y_marks=np.arange(ymin,ymax+interscale,interscale)
    print y_marks
    return y_marks
def draw_curve(x,y,width=512,height=512,title='title',xlabel='xlabel',ylabel='ylabel'):
    num=17
    ymarks=get_mark_value(y,num)
    xmarks=get_mark_value(x,num)
    ypmarks=get_mark_value(np.array((0.27,1.59)),num)
    xpmarks=get_mark_value(np.array((0.285,1.80)),num)
    x=norm(x,np.array([0,width]))
    y=norm(y,np.array([0,height]))
    x=x.astype('int32')
    y=y.astype('int32')
    #print x.dtype
    #print y.dtype
    img = np.zeros((height,width,3),np.uint8)#create a color none content img
    img=img+255 # make the image white 
    width_pre=int(width/(num-1)) 
    height_pre=int(height/(num-1)) 
    # draw the box 
    for i in range(0,num+1):
       for j in range(0,num+1):
           stx=i*width_pre
           sty=j*height_pre
           cv2.rectangle(img,(stx,sty),(width_pre,height_pre),(128,128,128),1)
    cv2.rectangle(img,(0,0), (width,height), (0,0,0),3)
    line_color=(0,0,255) # red
    # draw the line
    for i in range(0,len(x)):
        if i==0:
            cv2.line(img,(0,height-0),(x[i+1],height-y[i+1]),line_color,3)
        elif i==len(x)-1:
            print("draw line done")
        else:
            cv2.line(img,(x[i],height-y[i]),(x[i+1],height-y[i+1]),line_color,3)
    title_height=int(height*0.2)
    xlabel_width=int(width*0.2)
    text_scale=np.min([width,height])/350
    imgxlabel=draw_text(xlabel,width,title_height,text_scale)
    imgylabel=draw_text(ylabel,xlabel_width,height+title_height,text_scale,direc='v1')
    imgtitle=draw_text(title,width+xlabel_width,title_height,text_scale)
    imgvipkid=draw_text("vipkid bgdata group",xlabel_width,height+title_height*2,text_scale,(168,168,168),'v1')
    #print imgtitle.shape
    #print img.shape
    img=np.vstack((img,imgxlabel))    
    img=np.hstack((imgylabel,img))    
    img=np.vstack((imgtitle,img))
    img=np.hstack((img,imgvipkid))
    print "ymarks len=%d"%(len(ymarks))   
    for i in range(0,num):
        img=draw_mark(img,'%.2f'%(ymarks[i]),0.22,ypmarks[num-1-i])
        if i%2==0:
            img=draw_mark(img,'%.0f'%(xmarks[i]),xpmarks[i],1.55)

    #img=draw_mark(img,'%.2f'%(ymarks[i]),0.22,0.3)
    #img=draw_mark(img,'%.2f'%(ymin),0.22,1.5)
    #img=draw_mark(img,'%.2f'%(xmax),1.7,1.55)
    #img=draw_mark(img,'%.2f'%(xmin),0.33,1.55)
    save2server("roc{}.jpg".format(random.random()),img)
    return img

def append_all(x):
    x1=x[0]
    for xt in x:
        x1=np.append(x1,xt)
    return x1
def scale_all(x,a,b):
    for i in range(0,len(x)):
        x[i]=(x[i]-a)*b
    return x
def draw_line_new(x,y,img,line_color):
    height=512
    width=512
    x=x+float(width)*0.2
    y=y-float(height)*0.2
    x=x.astype('int')
    y=y.astype('int')
   
    #line_color=(0,0,255) # red
    # draw the line
    for i in range(0,len(x)):
        if i==0:
            cv2.line(img,(x[i],height-y[i]),(x[i+1],height-y[i+1]),line_color,1)
        elif i==len(x)-1:
            print("draw line done")
        else:
            cv2.line(img,(x[i],height-y[i]),(x[i+1],height-y[i+1]),line_color,1)
    return img 
def draw_curve_new(xl,yl,width=512,height=512,title='title',xlabel='xlabel',ylabel='ylabel'):
    num=17
    
    x=append_all(xl)
    y=append_all(yl)
    ymarks=get_mark_value(y,num)
    xmarks=get_mark_value(x,num)
    ypmarks=get_mark_value(np.array((0.27,1.59)),num)
    xpmarks=get_mark_value(np.array((0.285,1.80)),num)
    
    xn=norm(x,np.array([0,width]))
    yn=norm(y,np.array([0,height]))
    xn = (x-x.min())*float(width)/x.max()
    a=x.min()
    b=float(width)/x.max()
    xl=scale_all(xl,a,b)
    a=y.min()
    b=float(height)/y.max()
    yl=scale_all(yl,a,b)

    x=x.astype('int32')
    y=y.astype('int32')
    #print x.dtype
    #print y.dtype,
    img=draw_curve_bg(width,height,title,xlabel,ylabel)
    color=[0,0,0]
    for ii in range(0,len(xl)):
        color[ii%3]=255
        img=draw_line_new(xl[ii],yl[ii],img,tuple(color))
    print "ymarks len=%d"%(len(ymarks))   
    for i in range(0,num):
        img=draw_mark(img,'%.2f'%(ymarks[i]),0.22,ypmarks[num-1-i])
        if i%2==0:
            img=draw_mark(img,'%.0f'%(xmarks[i]),xpmarks[i],1.55)

    save2server("roc{}.jpg".format(random.random()),img)
    return img


def draw_curve_bg(width=512,height=512,title='title',xlabel='xlabel',ylabel='ylabel'):
    num=17
    img = np.zeros((height,width,3),np.uint8)#create a color none content img
    img=img+255 # make the image white 
    width_pre=int(width/(num-1)) 
    height_pre=int(height/(num-1)) 
    # draw the box 
    for i in range(0,num+1):
       for j in range(0,num+1):
           stx=i*width_pre
           sty=j*height_pre
           cv2.rectangle(img,(stx,sty),(width_pre,height_pre),(128,128,128),1)
    cv2.rectangle(img,(0,0), (width,height), (0,0,0),3)
    title_height=int(height*0.2)
    xlabel_width=int(width*0.2)
    text_scale=np.min([width,height])/350
    imgxlabel=draw_text(xlabel,width,title_height,text_scale)
    imgylabel=draw_text(ylabel,xlabel_width,height+title_height,text_scale,direc='v1')
    imgtitle=draw_text(title,width+xlabel_width,title_height,text_scale)
    imgvipkid=draw_text("vipkid bgdata group",xlabel_width,height+title_height*2,text_scale,(168,168,168),'v1')
    #print imgtitle.shape
    #print img.shape
    img=np.vstack((img,imgxlabel))    
    img=np.hstack((imgylabel,img))    
    img=np.vstack((imgtitle,img))
    img=np.hstack((img,imgvipkid))

    return img
def draw_curves(xlist,ylist):
    img=draw_curve(xlist[0],ylist[0])

def draw_mark(img,num_str,x_ratio,y_ratio):
    width=int(float(img.shape[1])*x_ratio)
    height=int(float(img.shape[0])*y_ratio)
    #width=x
    #height=y
    font = cv2.FONT_HERSHEY_SIMPLEX
    text_scale=0.4
    c=(200,10,10)
    direc='h'
    return putText(img, num_str, (width,height), font, text_scale, c, 1,direc)
def image2text(img,dst_size=(80,80)):
    # img2gray
    img_gray=cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
         
    # img2bw
    img_bw= image2bw(img_gray)
   
    # imgresize
    print img_bw.shape
    rows, cols = img_bw.shape
    img_rs = cv2.resize(img_bw, dst_size, interpolation=cv2.INTER_CUBIC)
    rows, cols = img_rs.shape

    # pixel 2 charactors
    text_pic=''
    im=img_rs
    save2server('tmp.jpg',im)
    for i in range(0,rows):
        for j in range(0,cols):
                 
                   if im[i,j] >128: # operate every pixel
                       text_pic+=' '
                   else:
                       text_pic+='#'
        text_pic+='\r\n'
    print str(text_pic)
    print im
    return text_pic



def add_random_value():
    for k in range(5000): #Create 5000 noisy pixels
        i = random.randint(0,im.height-1)
        j = random.randint(0,im.width-1)
        color = (random.randrange(256),random.randrange(256),random.randrange(256))
    img[i,j] = color    
    return img
def image2bw(GrayImage,method='adaptive_gaussian'):
    # mean filter 
    GrayImage= cv2.medianBlur(GrayImage,5)
    if method=='adaptive_gaussian': 
        th = cv2.adaptiveThreshold(GrayImage,255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C,cv2.THRESH_BINARY,3,5) 
    elif method=='adaptive_mean':#3->Block size, 5->param1  
        th = cv2.adaptiveThreshold(GrayImage,255,cv2.ADAPTIVE_THRESH_MEAN_C,cv2.THRESH_BINARY,3,5)
    else: 
        ret,th = cv2.threshold(GrayImage,127,255,cv2.THRESH_BINARY) 
    return th
def example_zmm():
    # read the original
    img = cv2.imread('../test2.jpg')
    #cv2.imshow('original', img)
    
    # expand
    rows, cols, channels = img.shape
    img_ex = cv2.resize(img, (2*cols, 2*rows), interpolation=cv2.INTER_CUBIC)
    #cv2.imshow('expand', img_ex)
    
    # zoom
    img_zo = cv2.resize(img, (cols/2, rows/2), interpolation=cv2.INTER_AREA)
    #cv2.imshow('zoom', img_zo)
    
    # trans
    M = np.array([[1, 0, 50],[0, 1, 50]], np.float32)
    cv2.imshow('trans', img_tr)
    #cv2.imshow('trans', img_tr)
    
    # Rotation
    M=cv2.getRotationMatrix2D((cols/2,rows/2), 45, 1)
    img_ro =cv2.warpAffine(img, M, img.shape[:2]) 
def getBluePixel(Img):
    HSV = cv2.cvtColor(Img, cv2.COLOR_BGR2HSV)
    H, S, V = cv2.split(HSV)
    LowerBlue = np.array([100, 100, 50])
    UpperBlue = np.array([130, 255, 255])
    mask = cv2.inRange(HSV, LowerBlue, UpperBlue)
    BlueThings = cv2.bitwise_and(Img, Img, mask=mask)
def test():
    x=np.array([-0.2,0.3,0.4,0.5])
    y=np.array([0.2,0.4,0.1,-0.4])
    norm(x,np.array([0,500]))
    
    img=draw_curve(x,y)
    #img=cv2.imread('../examples/faces/2007_007763.jpg')
    #print img.shape 
    #image2text(img,(80,40))
if __name__=='__main__':
    x=np.array([-0.2,0.3,0.4,0.5])
    y=np.array([0.2,0.4,0.1,-0.4])
    norm(x,np.array([0,500]))
    



    img=draw_curve(x,y)
    #img=cv2.imread('../examples/faces/2007_007763.jpg')
    print img.shape 
    image2text(img,(80,40))
    image2text(img,(80,40))



# -*- coding: utf-8 -*-
from vcvf import face_detector as fd
import cv2
import os




def predict(img_dir,reusltdir):
    
    fd1=fd.face_detector(1)
    files1 =os.listdir(img_dir)
    for xmlFile in files1:
        list1=[]
        name=xmlFile[:-4]
        imf=os.path.join(img_dir,xmlFile)
        print imf
        image=cv2.imread(imf)

        num,list1,time=fd1.detect_face(image)

        pathn=os.path.join(resultdir,xmlFile)
        pathn+=".txt"
        with open(pathn, 'w') as fp:
            fp.write("%s %d " % (name, num))
            for i in range(num):
                list2=str(list1[i])
                list2=list2[1:-1]
                print list2
                fp.write("%s " % list2)
    print 'vcvf_txt() is OK'

if __name__ == '__main__':
    img_dir="D:\zmm\moba\home\intern_mission\\bndbox_test2\\bndbox_test\\vcvf_txt\pre_file"
    resultdir="D:\zmm\moba\home\intern_mission\\bndbox_test2\caculate_roc\\predict_result"
    predict(img_dir,resultdir)



from distutils.core import setup
from setuptools import setup
from setuptools import find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()
setup(
    name="ai_tools",
    version="0.0.8",
    description="basic tools for ai",
    long_description=long_description,
    long_description_content_type="text/markdown",
    


    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    #packages=find_packages(exclude=['contrib', 'docs', 'tests']),  # Required
    #packages=find_packages(include=['contrib', 'docs', 'ai_tools']),  # Required
    packages=find_packages(),  # Required
    #test_suite='tests',
    #py_modules=["ai_tools_draw"],
    #py_modules=["ai_tools/ai_tools_save2server.py"],
    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    #install_requires=['opencv'],  # Optional

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={  # Optional
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #
    # If using Python 2.6 or earlier, then these have to be included in
    # MANIFEST.in as well.
    package_data={  # Optional
        'package': ['package/data.dat'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[('my_data', ['data/datafile'])],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    entry_points={  # Optional
        'console_scripts': [
            'sample=sample:main',
        ],
    },




    #scripts = ['drawlib.py','save2server.py'], 
    license = "MIT",
    author="zhaomingming",
    author_email="13271929138@163.com",
    url="http://www.zhaomingming.cn",
    #py_modules=['ai_tools'],
    platforms = 'any'
)



# -*- coding: utf-8 -*-
import xml.dom.minidom
import os

def txt2xml(xmlfilename,txtfilename):
    with open(xmlfilename,'r') as fs:
            managerList=[]
            doc = xml.dom.minidom.Document()
            root = doc.createElement('Recognition')
            root.setAttribute('type', 'face')
            doc.appendChild(root)

            str=fs.read()
            line=str.split()

            for i in range(0,int(line[1])):
                managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])

            for i in managerList :
                for j in range(len(i)):
                    nodeManager = doc.createElement('bndbox')
                    nodeXmin = doc.createElement("xmin")
                    nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                    nodeYmin = doc.createElement("ymin")
                    nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                    nodeXmax = doc.createElement("xmax")
                    nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                    nodeYmax = doc.createElement("ymax")
                    nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))


                    nodeManager.appendChild(nodeXmin)
                    nodeManager.appendChild(nodeYmin)
                    nodeManager.appendChild(nodeXmax)
                    nodeManager.appendChild(nodeYmax)
                    root.appendChild(nodeManager)

            print line[0]
            #pathn=os.path.join(after_file,line[0])
            #pathn=os.path.join(pathn,".xml")
            #pathn+=".xml"
            #fp = open(pathn, 'w')
            fp = open(txtfilename, 'w')
            doc.writexml(fp, indent='\t', addindent='\t', newl='\n', encoding="utf-8")


def Transform_txt_xml(pre_file='txt_file',after_file='xml_file'):
    line=[]
    
    files1 =os.listdir(pre_file+"//*.txt")
    for xmlFile in files1:
        with open(os.path.join(pre_file,xmlFile),'r') as fs:
            managerList=[]
            doc = xml.dom.minidom.Document()
            root = doc.createElement('Recognition') 
            root.setAttribute('type', 'face') 
            doc.appendChild(root)
            
            str=fs.read()
            line=str.split()
             
            for i in range(0,int(line[1])):
                managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])
            
            for i in managerList :
                for j in range(len(i)):
                    nodeManager = doc.createElement('bndbox')
                    nodeXmin = doc.createElement("xmin")
                    nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                    nodeYmin = doc.createElement("ymin")
                    nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                    nodeXmax = doc.createElement("xmax")
                    nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                    nodeYmax = doc.createElement("ymax")
                    nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))
        
                 
                    nodeManager.appendChild(nodeXmin)
                    nodeManager.appendChild(nodeYmin)
                    nodeManager.appendChild(nodeXmax)
                    nodeManager.appendChild(nodeYmax)
                    root.appendChild(nodeManager)
    
            print line[0]
            pathn=os.path.join(after_file,line[0])
            #pathn=os.path.join(pathn,".xml")
            pathn+=".xml"
            fp = open(pathn, 'w')
            doc.writexml(fp, indent='\t', addindent='\t', newl='\n', encoding="utf-8")
    print 'Transform_txt_xml() is OK'
        
if __name__ == '__main__':
    Transform_txt_xml()
            
    




import os
import random


def logw(fh,message):
    print(message)
    fh.write(message)
def create_score_vector(num):
    
    fh=open("score_vector.txt","w")
    for i in range(0,num):
        sample_type=-1
        if random.random()>0.5:
            sample_type=1
        
        score=random.random()
        logw(fh,"%d %f\n"%(sample_type,score))
        
    fh.close()
create_score_vector(100)



# -*- coding: utf-8 -*-
import pylab as pl
from Analyze_txt import Analyze_txt
import sys
from ai_tools import draw
from ai_tools import save2server
def txt2xml(xmlfilename,txtfilename):
    with open(xmlfilename,'r') as fs:
            managerList=[]
            doc = xml.dom.minidom.Document()
            root = doc.createElement('Recognition')
            root.setAttribute('type', 'face')
            doc.appendChild(root)

            str=fs.read()
            line=str.split()
            print line
            for i in range(0,int(line[1])):
                managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])

            for i in managerList :
                for j in range(len(i)):
                    nodeManager = doc.createElement('bndbox')
                    nodeXmin = doc.createElement("xmin")
                    nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                    nodeYmin = doc.createElement("ymin")
                    nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                    nodeXmax = doc.createElement("xmax")
                    nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                    nodeYmax = doc.createElement("ymax")
                    nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))


                    nodeManager.appendChild(nodeXmin)
                    nodeManager.appendChild(nodeYmin)
                    nodeManager.appendChild(nodeXmax)
                    nodeManager.appendChild(nodeYmax)
                    root.appendChild(nodeManager)

            print line[0]
            #pathn=os.path.join(after_file,line[0])
            #pathn=os.path.join(pathn,".xml")
            #pathn+=".xml"
            #fp = open(pathn, 'w')
            fp = open(txtfilename, 'w')
            doc.writexml(fp, indent='\t', addindent='\t', newl='\n', encoding="utf-8")

def roc(standard_path="truth",test_path="test",result_roc='roc.txt',result_jpg='roc.jpg'):

    db=[]
    print "analyze_xml:"
    db,pos=Analyze_txt(standard_path,test_path)
    db = sorted(db, key=lambda x:x[2], reverse=True)#sorted() 
    
    print pos 
    xy_arr = []
    tp,fp = 0., 0.			
    for i in range(len(db)):
        #print db
        fp += db[i][1]#wrong
        tp += db[i][0]#creat
        xy_arr.append([fp,tp/pos,db[i][2]])
   

    auc = 0.			
    prev_x = 0
    for x,y,t in xy_arr:
	    if x != prev_x:
		    auc += (x - prev_x) * y
		    prev_x = x
            
    print xy_arr[-1]        
    x = [_a[0] for _a in xy_arr]
    y = [_a[1] for _a in xy_arr]
    y1 = [_a[2] for _a in xy_arr]
    #pl.title("ROC (AUC = %.4f)" % (auc/float(x[-1])))
    #pl.xlabel("False Count")
    #pl.ylabel("True Positive Rate")
    #pl.plot(x, y)
    #pl.show()
    xl=[x,x]
    yl=[y,y1]
    img=draw.draw_curve_new(xl,yl,width=512,height=512,title='ROC (AUC=%.4F)'%(auc/float(x[-1])),xlabel='FalseAlarm Count',ylabel='True Positive Rate') 
    save2server.save2server(result_jpg,img)
    with open(result_roc, 'w') as fp:
        for i in range(len(xy_arr)):
            fp.write("%f %f %f \n" % (xy_arr[i][0], xy_arr[i][1],xy_arr[i][2]))
    
if __name__ == '__main__':
    gt_dir=sys.argv[1]
    resultdir=sys.argv[2]
    print gt_dir
    print resultdir
    roc(gt_dir,resultdir,'result_roc.txt',"roc.jpg")
